@@ -38,6 +38,9 @@ Puppet::Type.type(:keystone_user).provide(
     if resource[:password]
       properties << '--password' << resource[:password]
     end
+    if resource[:description]
+      properties << '--description' << resource[:description]
+    end
     if resource[:email]
       properties << '--email' << resource[:email]
     end
@@ -63,6 +66,7 @@ Puppet::Type.type(:keystone_user).provide(
       options << '--disable' if @property_flush[:enabled] == :false
       # There is a --description flag for the set command, but it does not work if the value is empty
       options << '--password' << resource[:password] if @property_flush[:password]
+      options << '--description' << resource[:description] if @property_flush[:description]
       options << '--email'    << resource[:email]    if @property_flush[:email]
       # project handled in tenant= separately
       unless options.empty?
@@ -98,6 +102,10 @@ Puppet::Type.type(:keystone_user).provide(
     @property_flush[:enabled] = value
   end
 
+  def description=(value)
+    @property_flush[:description] = value
+  end
+
   def email=(value)
     @property_flush[:email] = value
   end
@@ -121,17 +129,6 @@ Puppet::Type.type(:keystone_user).provide(
       # will know we are doing v3password auth - otherwise, it is not used.  The
       # user_id uniquely identifies the user including domain.
       credentials.username = resource[:name]
-      # Need to specify a project id to get a project scoped token.  List
-      # all of the projects for the user, and use the id from the first one.
-      projects = self.class.request('project', 'list', ['--user', id, '--long'])
-      if projects && projects[0] && projects[0][:id]
-        credentials.project_id = projects[0][:id]
-      else
-        # last chance - try a domain scoped token
-        credentials.domain_id = domain_id
-      end
-
-      credentials.identity_api_version = '2' if credentials.auth_url =~ /v2\.0\/?$/
 
       begin
         token = Puppet::Provider::Openstack.request('token', 'issue', ['--format', 'value'], credentials)
